/**
 * @用途
 * @author 黄敏
 * @创建时间 2022-08-08 08:39
 **/

import Dexie from 'dexie'

let version = 1

//统一数据格式
interface CommonData {
  id: string | number //唯一id
  content: any
  version: number //版本
  expiration: number //过期时间
}

//联系人数据
interface ContactData extends CommonData {
  content: Crawler.Contact //数据格式
}

//公司数据
interface CompanyData extends CommonData {
  content: Crawler.Company //数据格式
}

//综合数据
interface ResultData extends CommonData {
  content: Crawler.Result
}

//好友验证信息
interface ValidationFriend extends CommonData {
  content: Crawler.FriendInfo
}

interface MutualFriendsData extends CommonData {
  content: Crawler.MutualFriends //数据格式
}

class CrawlerDatabase extends Dexie {
  public contact: Dexie.Table<ContactData, string | number>
  public company: Dexie.Table<CompanyData, string | number>
  public result: Dexie.Table<ResultData, string | number>
  public validationFriend: Dexie.Table<ValidationFriend, string>
  public mutualFriends: Dexie.Table<MutualFriendsData, string | number> //linkedin/facebook共同好友

  //共用缓存
  public temp: Dexie.Table<CommonData, string | number>

  public constructor(databaseName: string) {
    super(databaseName)
    this.version(1).stores({
      contact: 'id', //联系人缓存
      company: 'id', //公司信息缓存
      result: 'id', //其它缓存
      temp: 'id', //其它缓存
      validationFriend: 'id',
      mutualFriends: 'id'
    })

    this.contact = this.table('contact')
    this.company = this.table('company')
    this.result = this.table('result')
    this.temp = this.table('temp')
    this.validationFriend = this.table('validationFriend')
    this.mutualFriends = this.table('mutualFriends')
  }
}

class CrawlerDb {
  private db: CrawlerDatabase

  //是否禁用缓存
  private isDisable:boolean = false;

  //禁用数据库
  disableDb(){
    this.isDisable = true;
  }

  //启用数据库
  enableDb(){
    this.isDisable = false;
  }
  constructor() {
    this.db = new CrawlerDatabase(`ff-crawler-v1`)
    if (['complete', 'interactive'].includes(document.readyState)) {
      console.log('已经加载完')
      this.opendb()
    } else {
      console.log('要等待')
      window.addEventListener('DOMContentLoaded', () => {
        console.log('等待，完成')
        this.opendb()
      })
    }
  }

  opendb() {
    this.db
      .open()
      .then(() => {
        console.log('爬虫数据库开启成功')
      })
      .catch(function (e) {
        console.log('爬虫数据库开启出错: ' + e.stack)
      })
  }


  /**
   * 保存联系人信息
   * @param id {String} 唯一id ，一般使用urlId或url
   * @param {mixed} contact 存入数据存的内容
   * @param {string} type 类型，如linkedin、facebook、等
   * @return 无返回值
   * */
  saveContact(id: string | number, content: Crawler.Contact, type = 'default') {
    let expiration = ff.util.scopeRandom(172800000, 345600000) //有效期2~4天内的随机数
    this.db.contact
      .put({
        id: type + '-' + id,
        content: content,
        version: version,
        expiration: expiration + Date.now()
      })
      .then(function () {
        console.log('写入成功')
      })
      .catch(e => {
        console.error('saveContactInfo error', e)
      })
  }

  /**
   * 从本地获取联系人信息
   * @param {String} id 获取指定id的内容，id一般为urlid或url
   * @param {String} type 数据类型，有值：default|linkedin|facebook|twitter|google
   * @return {Promise(Crawler.Contact | null)}
   * */
  async getContact(id: string | number, type = 'default'): Promise<Crawler.Contact | null> {
    try {

      if (this.isDisable) return null

      let data = await this.db.contact.get(type + '-' + id)

      //数据库版本必须一致
      if (data && data.version === version && data.expiration > Date.now()) {
        return data.content
      } else {
        if (data) {
          await this.db.contact.delete(type + '-' + id)
        }
        return null
      }
    } catch (e) {
      console.error('getContact error', e, id)
      throw e
    }
  }

  /**
   * 保存公司信息
   * @param id {String} 唯一id ，一般使用urlId或url
   * @param type {String} 类型，可选linkedin facebook twitter default
   * @param {mixed} contact 存入数据存的内容
   * @return void
   * */
  saveAccount(id: number | string, company: Crawler.Company, type = 'default', ext?: any | null): void {
    let expiration = ff.util.scopeRandom(86400000, 172800000) //有效期1~5天内的随机数,公司信息比较少变化
    if (ext) Object.assign(company, ext)
    this.db.company
      .put({
        id: type + '-' + id,
        content: company,
        version: version,
        expiration: expiration + Date.now()
      })
      .then(function (e) {
        console.log('写入成功')
      })
      .catch(e => {
        console.error('saveCompany error', e)
      })
  }

  /**
   * 从本地获取公司信息
   * @param {String} id 获取指定id的内容，id一般为urlid或url
   * @param {String} type 数据类型，有值：default|linkedin|facebook|twitter|google
   * @return {Promise(Crawler.Company | null)}
   * */
  async getAccount(id: number | string, type = 'default'): Promise<Crawler.Company | null> {
    try {

      if (this.isDisable) return null

      let data = await this.db.company.get(`${type}-${id}`)
      //数据库版本必须也要一致
      if (data && data.version === version && data.expiration > Date.now()) {
        return data.content
      } else {
        if (data) {
          await this.db.company.delete(id)
        }
        return null
      }
    } catch (e) {
      console.error('getCompany error', e, id)
      throw e
    }
  }

  /**
   * 从本地获取综合数据
   * @param {String} id 获取指定id的内容，id一般为urlid或url
   * @param {String} type 数据类型，有值：default|linkedin|facebook|twitter|google
   * @return {Promise(Crawler.Company | null)}
   * */
  async getResult(id: number | string, type = 'default'): Promise<Crawler.Result | null> {
    try {
      if (this.isDisable) return null
      let data = await this.db.result.get(`${type}-${id}`)
      //数据库版本必须也要一致
      if (data && data.version === version && data.expiration > Date.now()) {
        return data.content
      } else {
        if (data) {
          await this.db.company.delete(id)
        }
        return null
      }
    } catch (e) {
      console.error('getCompany error', e, id)
      throw e
    }
  }

  /**
   * 保存公司信息
   * @param id {String} 唯一id ，一般使用urlId或url
   * @param type {String} 类型，可选linkedin facebook twitter default
   * @param {mixed} contact 存入数据存的内容
   * @return void
   * */
  saveResult(id: number | string, result: Crawler.Result, type = 'default'): void {
    let expiration = ff.util.scopeRandom(86400000, 172800000) //有效期1~5天内的随机数,公司信息比较少变化

    this.db.result
      .put({
        id: type + '-' + id,
        content: result,
        version: version,
        expiration: expiration + Date.now()
      })
      .then(function (e) {
        console.log('写入成功')
      })
      .catch(e => {
        console.error('saveCompany error', e)
      })
  }

  /*
   * 保存缓存至本地,默认缓存1天时间
   * @param id {String} 唯一id ，一般使用urlId或url
   * @param {mixed} contact 存入数据存的内容
   * @param {Inte} day 可选，默认缓存一天
   * @return 无返回值
   * */
  saveTemp(id: number | string, content: any, day = 1) {
    let expiration
    if (day > 0) {
      expiration = Date.now() + 86400000 * day
    } else {
      expiration = Infinity
    }
    this.db.temp
      .put({
        id: id,
        content: content,
        version: version,
        expiration: expiration
      })
      .then(function () {
        console.log('保存数据库成功')
      })
      .catch(e => {
        console.error('saveTemp error', e)
      })
  }

  /* 获取本地缓存
   * @param {String} id 获取指定id的内容，id一般为urlid或url
   * @return Promise(data) 无数据值返回null
   * */
  async getTemp(id: string | number) {
    try {

      if (this.isDisable) return null

      let data = await this.db.temp.get(id)

      if (data && data.version === version && data.expiration > Date.now()) {
        return data.content
      } else {
        if (data) {
          await this.db.temp.delete(id)
        }

        return null
      }
    } catch (e) {
      console.error('getTemp error', e, id)
      throw e
    }
  }

  /**
   * 获取是否为朋友关系
   * @param uid
   * @param uuid暂时不用加uuid
   * */
  async getValidationFriend(uid: string, type: f2.info.Type, uuid: number | string = 0): Promise<Crawler.FriendInfo | null> {
    if (this.isDisable) return null
    let data = await this.db.validationFriend.get(`${type}-${uid}-${uuid}`)
    if (!data || data.expiration < Date.now() || data.version !== version) {
      return null
    }
    return data.content
  }

  async saveValidationFriend(uid: string, type: f2.info.Type, friend: Crawler.FriendInfo, uuid: number | string = 0) {
    await this.db.validationFriend.put({
      id: `${type}-${uid}-${uuid}`,
      expiration: 600000 + Date.now(), //保存10分钟
      version: version,
      content: friend
    })
  }

  async delValidationFriend(uid: string, type: f2.info.Type) {
    await this.db.validationFriend.delete(type + '-' + uid)
  }

  /**
   * 联系人与操作员（CM）的共同好友
   * @param id {String} 唯一id ，一般使用urlId或url
   * @param {mixed} contact 存入数据存的内容
   * @return 无返回值
   * */
  saveMutualFriends(id: string | number, content: Crawler.MutualFriends, type = 'default') {
    let expiration = ff.util.scopeRandom(172800000, 345600000) //有效期2~4天内的随机数
    this.db.mutualFriends
      .put({
        id: type + '-' + id,
        content: content,
        version: version,
        expiration: expiration + Date.now()
      })
      .then(function () {
        console.log('murual-friend 写入成功')
      })
      .catch(e => {
        console.error('saveContactInfo error', e)
      })
  }

  async getMutualFriends(id: string | number, type = 'default'): Promise<Crawler.MutualFriends | null> {
    if (this.isDisable) return null
    try {
      let data = await this.db.mutualFriends.get(type + '-' + id)
      //数据库版本必须一致
      if (data && data.version === version && data.expiration > Date.now()) {
        return data.content
      } else {
        if (data) {
          await this.db.mutualFriends.delete(type + '-' + id)
        }
        return null
      }
    } catch (e) {
      console.error('getContact error', e, id)
      return Promise.reject(e)
    }
  }
}

export default CrawlerDb
