//联系人/公司通用的必填项

interface CompanyRequired {
	origin: string
	update_time: number
	plugin_id: string
	name: string
	completed: boolean
}

interface ContactRequired {
	origin: string
	update_time: number
	plugin_id: string
	completed: boolean
}

interface BatchRquire {
	api: string
	origin: string
	title: string
	uid?: string
	total?: number
}

interface OriginData {
	origin: string
	type: Crawler.DataType
	UID: string
}

interface PooledDataParams {
	company: Crawler.Company[]
	contact: Crawler.Contact[]
}

class CrawlerUtil {
	readonly uid_separator = '|#|'

	//爬虫公司模板
	companyModel(require: CompanyRequired): Crawler.Company {
		let model: Crawler.Company = {
			origin: require.origin,
			update_time: 0,
			plugin_id: '',
			country: '',
			name: '',
			completed: true,
			aliases: [], //别名
			founders: [], //创始人
			industrys: [], //行业,一个数组，可以有多个行业
			specialties: [], //领域特长
			as_numbers: [], //as_numbers号,可以有多个as号
			domains: [], // 域名,域名
			addresses: [], //地址
			infos: [],
			imgs: [], // 图片
			stock: [], // 股票数据
			partners: [], // 合伙人
			godaddy: false,
			registeredCapital: '', //注册资本 数字
			currencyUnit: '', //注册单位
			registeredTime: 0, //公司注册时间
			remark: ''
		}

		return Object.assign(model, require)
	}

	//爬虫公司模板
	contactModel(require: ContactRequired): Crawler.Contact {
		let model: Crawler.Contact = {
			origin: require.origin,
			update_time: 0, //采集时间+
			completed: true, //数据是否完整
			plugin_id: '', //旧版用于验证的id
			contacts_names: [],
			// name: '',			//联系人名称
			// first_name: '',
			// last_name: '',
			birth_year: 0, //生日年份
			birth_month: 0, //生日月份
			birth_day: 0, //生日天
			addresses: [], //地址
			educations: [], //教育
			works: [], //工作经历
			imgs: [], //图片
			infos: [],
			medias: []
		}

		return Object.assign(model, require)
	}

	/**
	 * 获取来源数据id,该id是唯一的，每个数据来源都会有一个唯一的id，
	 * @ param {string} id 页面id,唯一的依据，也有可能是一个网址
	 * @ param {enum} origin 来源 ['linkedin', 'twitter', 'facebook', 'crunchbase', 'url']，
	 * 						linkedin/twitter/facebook/crunchbase的id是urlid,url的id是url
	 * @ param {boolean} isCompany 是否是公司数据
	 * linkedin|#|contact|#|2q34sdf2rf233f2,facebook|#|contact|2q34sdf2rf233f2
	 */
	createUniqueId(id: string | number, origin: string, type: Crawler.DataType = 'contact') {
		if (!id || !origin) {
			throw `生成origin id,必须要有id(页面唯一id)和origin(来源),当前值：UID:${id},origin:${origin}`
		}

		return `${origin}${this.uid_separator}${type}${this.uid_separator}${id}`
	}

	//是否是UniqueId
	isUniqueId(id: string): boolean {
		if (!id || !ff.util.isString(id)) return false
		let arr: string[] = id.split(this.uid_separator)
		//UniqueId只有两或三个部分
		return arr.length === 3
	}

	/*
	 * 解析unique id
	 * @param {String} originId
	 * @return {Object} {origin:linkedin|twitter|facebook|url,UID:'id',isCompany:true|false}
	 * */
	decodeUniqueId(uniqueId: string): OriginData {
		if (!this.isUniqueId(uniqueId)) throw `解析uniqueId失败 格式不正确: ${uniqueId}`

		let arr = uniqueId.split(this.uid_separator)
		let length = arr.length

		let returns: OriginData

		if (length === 3) {
			returns = {
				origin: arr[0],
				type: <Crawler.DataType>arr[1],
				UID: <string>arr[2]
			}
		} else {
			throw `解析origin_id失败 格式不正确: ${uniqueId}`
		}

		return returns
	}

	//通过OID,获取有效链接
	getUniqueIdLink(uniqueId: string): string {
		let url = ''

		if (!uniqueId || !this.isUniqueId(uniqueId)) return url

		let codes = this.decodeUniqueId(uniqueId)

		if (!codes) {
			return url
		} else if (codes.origin === 'linkedin') {
			if (codes.type === 'contact') {
				url = `https://www.linkedin.com/in/${codes.UID}`
			} else if (codes.type === 'company') {
				url = `https://www.linkedin.com/company/${codes.UID}`
			}
		} else if (codes.origin === 'facebook') {
			url = `https://www.facebook.com/${codes.UID}`
		} else if (codes.origin === 'twitter') {
			url = `https://twitter.com/${codes.UID}`
		} else if (codes.origin === 'crunchbase') {
			url = `https://www.crunchbase.com/organization/${codes.UID}`
		} else if (codes.origin === 'rocketreach') {
			if (codes.type === 'contact') {
				url = `https://rocketreach.co/${codes.UID}`
			} else if (codes.type === 'company') {
				url = `https://rocketreach.co/v2/companyDetails?id=${codes.UID}`
			}
		} else if (codes.origin === 'tianyancha') {
			if (/^\d+/.test(codes.UID)) {
				url = `https://www.tianyancha.com/company/${codes.UID}`
			} else {
				url = `https://www.tianyancha.com/brand/${codes.UID}`
			}
		} else if (codes.origin === 'wsj') {
			if (codes.type === 'contact') {
				url = `https://www.wsj.com/market-data/quotes/${codes.UID}`
			} else if (codes.type === 'company') {
				url = `https://www.wsj.com/market-data/quotes/${codes.UID}/company-people`
			}
		}

		return url
	}

	/**
	 * 获取某个网站的某个cookie值
	 * @param {string} name cookie的键（名称）
	 * @param domain 网站的域名
	 * */
	async getCookie(name: string, domain: string) {
		let args = domain || name ? {domain: domain, name: name} : {}
		let e = await _pre_.getCookie(args)
		if (e.length) {
			return e[0].value
		} else {
			return null
		}
	}

	async setCookie(data:Electron.CookiesSetDetails){
		await _pre_.setCookie(data)
	}

	/**
	 * 获取某个网站的cookie字符串
	 * @param {string} domain 网站的域名
	 * */
	async getDomainCookies(domain: string) {
		let cookie = await _pre_.getCookie( {domain: domain})
		if (cookie.length) {
			return cookie.map(v => `${v.name}=${v.value}`).join('; ')
		} else {
			return ''
		}
	}

	//获取extContact 公司数据
	extContactToExtCompany(extContact: Crawler.Contact): Crawler.Company {
		let model = ff.crawler.util.companyModel({
			origin: extContact.origin,
			update_time: extContact.update_time,
			completed: true,
			plugin_id: extContact.plugin_id,
			name: extContact.contacts_names[0].first_name + ' ' + extContact.contacts_names[0].last_name
		})

		model.geo_region = extContact.geo_region
		model.country = extContact.country ?? ''
		model.remark = extContact.remark
		model.partners = []

		model.addresses = extContact.addresses.map(v => ({
			country: v.address.length === 2 ? v.address : '',
			address: v.address,
			longitude: v.longitude,
			latitude: v.latitude
		}))

		model.imgs = extContact.imgs.map(v => ({url: v, type: 'logo'}))
		model.infos = JSON.parse(JSON.stringify(extContact.infos))

		return model
	}

	/**
	 * 合并两个联系人
	 * @param {Crawler.Contact} origin 原
	 * @param {Crawler.Contact} newc 新的
	 * */
	mergeContact(origin: Crawler.Contact, newc: Crawler.Contact) {
		if (newc.remark) {
			if (!origin.remark || newc.remark.includes(origin.remark)) origin.remark = newc.remark
			else origin.remark += ' ' + newc.remark
		}

		if (!origin.birth_year && newc.birth_year) origin.birth_year = newc.birth_year
		if (!origin.birth_month && newc.birth_month) origin.birth_month = newc.birth_month
		if (!origin.birth_day && newc.birth_day) origin.birth_day = newc.birth_day
		if (!origin.geo_region && newc.geo_region) origin.geo_region = newc.geo_region
		if (!origin.country && newc.country) origin.country = newc.country
		if (!origin.remark && newc.remark) origin.remark = newc.remark

		//合并名字
		newc.contacts_names.forEach(v => {
			if (!origin.contacts_names.some(n => n.first_name === v.first_name && n.last_name === v.last_name)) {
				origin.contacts_names.push(v)
			}
		})

		//合并地址
		if (origin.addresses?.length) {
			if (newc.addresses) {
				newc.addresses.forEach(noe => {
					let i = origin.addresses.findIndex(f => f.address === noe.address || noe.address.indexOf(f.address) > -1)
					if (i > -1) {
						if (!origin.addresses[i].latitude && noe.latitude) origin.addresses[i].latitude = noe.latitude
						if (!origin.addresses[i].longitude && noe.longitude) origin.addresses[i].longitude = noe.longitude
					} else {
						origin.addresses.push(noe)
					}
				})
			}
		} else if (newc.addresses) {
			origin.addresses = newc.addresses
		}
		//教育经历
		if (origin.educations.length) {
			if (newc.educations) {
				newc.educations.forEach(noe => {
					let i = origin.educations.findIndex(f => f.college === noe.college || noe.college.indexOf(f.college) > -1)
					if (i > -1) {
						//如果新数据有origin_id而原数据没有，用新数据
						if (!origin.educations[i].start_date && noe.start_date) origin.educations[i].start_date = noe.start_date
						if (!origin.educations[i].end_date && noe.end_date) origin.educations[i].end_date = noe.end_date
						if (!origin.educations[i].degree_name && noe.degree_name) origin.educations[i].degree_name = noe.degree_name
						if (!origin.educations[i].study && noe.study) origin.educations[i].study = noe.study
						if (!origin.educations[i].descr && noe.descr) origin.educations[i].descr = noe.descr
					} else {
						origin.educations.push(noe)
					}
				})
			}
		} else if (newc.educations) {
			origin.educations = newc.educations
		}

		if (origin.works.length) {
			if (newc.works) {
				newc.works.forEach(noe => {
					let i = origin.works.findIndex(f => {
						return (f.company === noe.company || noe.company.indexOf(f.company) > -1 || f.company.indexOf(noe.company) > -1) && f.positions === noe.positions
						// || noe.positions.indexOf(f.positions) > -1 || f.positions.indexOf(noe.positions)>-1
					})
					if (i > -1) {
						//如果新数据有origin_id而原数据没有，用新数据
						if (!origin.works[i].location && noe.location) origin.works[i].location = noe.location
						if (!origin.works[i].positions && noe.positions) origin.works[i].positions = noe.positions
						if (!origin.works[i].descr && noe.descr) origin.works[i].descr = noe.descr
						if (!origin.works[i].start_date && noe.start_date) origin.works[i].start_date = noe.start_date
						if (!origin.works[i].end_date && noe.end_date) origin.works[i].end_date = noe.end_date
						if (!origin.works[i].unique_id && noe.unique_id) origin.works[i].unique_id = noe.unique_id
					} else {
						origin.works.push(noe)
					}
				})
			}
		} else if (newc.works.length) {
			origin.works = newc.works
		}

		newc.infos.forEach(info => {
			if (!origin.infos.some(v => v.type === info.type && v.info === info.info)) {
				origin.infos.push(info)
			}
		})
	}

	//两个联系人是否是同一个联系人
	isSameContact(d1: Crawler.Contact, d2: Crawler.Contact) {
		if (ff.util.sameText(d1.contacts_names.map(v => v.first_name + ' ' + v.last_name).join(' | '), d2.contacts_names.map(v => v.first_name + ' ' + v.last_name).join(' | '), false, true)) {
			return true
		}

		if (d1.infos.length && d2.infos.length) {
			//只要有一个联系方式是一样，那就是同一个公司
			let f1s = d1.infos,
					f2s = d2.infos
			if (f1s.some(f1 => f2s.some(f2 => f1.type !== 'phone' && f1.type === f2.type && ff.util.sameText(String(f1.info), String(f2.info))))) {
				return true
			}
		}

		return false
	}

	//两个公司是否是同一个公司
	isSameCompany(d1: Crawler.Company, d2: Crawler.Company) {
		if (ff.util.sameText(d1.name, d2.name, false, true)) {
			return true
		}

		if (d1.infos.length && d2.infos.length) {
			//只要有一个联系方式是一样，那就是同一个公司
			let f1s = d1.infos,
					f2s = d2.infos
			if (f1s.some(f1 => f2s.some(f2 => f1.type === f2.type && ff.util.sameText(String(f1.info), String(f2.info))))) {
				return true
			}
		}

		if (d1.domains.length && d2.domains.length) {
			if (d1.domains.some(str1 => d2.domains.some(str2 => ff.util.sameText(str1, str2, false, true)))) {
				return true
			}
		}

		return false
	}

	/**
	 * 数据去重合并，判断出为同一个联系人时，会自动合并去重
	 * @param {Array[Object]} datas 等待去重的数据数组
	 * @param {Crawler.Contact[]} originData 其它数据,datas会先与之对比，如果是同一人，那优先合到originData字段，默认为空数组[]
	 * @returns {Array[Object]} 返回非originData,合并后的数据
	 * */

	pooledContactData(datas: Crawler.Contact[], originData: Crawler.Contact[] = []) {
		let newDates: Crawler.Contact[] = []
		//	联系人数据
		for (let newc of datas) {
			let bool = originData.some(origin => {
				if (this.isSameContact(origin, newc)) {
					this.mergeContact(origin, newc)
					return true
				}
			})

			if (!bool) {
				bool = newDates.some(origin => {
					if (this.isSameContact(origin, newc)) {
						this.mergeContact(origin, newc)

						return true
					}
				})
			}

			if (!bool) {
				newDates.push(newc)
			}
		}
		return newDates
	}

	/**
	 * 数据去重合并，判断出为同一个公司时，会自动合并去重
	 * @param {Crawler.Company[]} datas 需要合并的公司数据
	 * @param {boolean}   forcedMergers 是否将datas强制合并成一个公司
	 * @returns {Crawler.Company[]} 返回自动合并后的数据
	 * */
	pooledCompanyData(datas: Crawler.Company[], forcedMergers = false) {
		let newDates: Crawler.Company[] = []
		//公司数据
		for (let d of datas) {
			let bool = newDates.some(n => {
				if (forcedMergers || (d && this.isSameCompany(n, d))) {
					if (!n.aliases) n.aliases = []
					if (!d.aliases) d.aliases = []

					n.aliases = ff.util.arrayUnique([...n.aliases, ...d.aliases])

					//合并创始人，

					if (n.founders && n.founders.length) {
						let nf = n.founders
						if (d.founders) {
							d.founders.forEach(noe => {
								let i = nf.findIndex(f => f.name === noe.name)
								if (i > -1) {
									//如果新数据有origin_id而原数据没有，用新数据
									if (!nf[i].origin_id && noe.origin_id) {
										nf[i].origin_id = noe.origin_id
									}
								} else {
									nf.push(noe)
								}
							})
						}
					} else if (d.founders) {
						n.founders = d.founders
					}

					if (d.remark) {
						if (!n.remark || d.remark.includes(n.remark)) n.remark = d.remark
						else n.remark += ' ' + d.remark
					}

					if (!n.founded && d.founded) n.founded = d.founded

					if (!n.business_hours && d.business_hours) n.business_hours = d.business_hours
					if (!n.country && d.country) n.country = d.country
					if (!n.geo_region && d.geo_region) n.geo_region = d.geo_region
					if (!n.geo_region && d.geo_region) n.geo_region = d.geo_region
					n.industrys = ff.util.arrayUnique([...n.industrys, ...d.industrys])
					n.specialties = ff.util.arrayUnique([...n.specialties, ...d.specialties])
					n.as_numbers = ff.util.arrayUnique([...n.as_numbers, ...d.as_numbers])
					n.as_numbers = ff.util.arrayUnique([...n.as_numbers, ...d.as_numbers])
					if (!n.employees && d.employees) n.employees = d.employees
					if (!n.staffCount && d.staffCount) n.staffCount = d.staffCount
					if (!n.staffCount && d.staffCount) n.staffCount = d.staffCount

					d.infos.forEach(info => {
						let bool = n.infos.some(v => {
							if (v.type === info.type && (v.info === info.info || (v.unique_id && v.unique_id === info.unique_id))) {
								if (info.origin_address) v.origin_address = info.origin_address
								if (info.origin_name) v.origin_name = info.origin_name
								return true
							} else {
								return false
							}
						})

						if (!bool) {
							n.infos.push(info)
						}
					})

					n.domains = ff.util.arrayUnique([...n.domains, ...d.domains])

					d.domains.forEach(domain => {
						let bool = n.domains.every(d2 => d2 !== domain && domain.indexOf(d2) === -1 && d2.indexOf(domain) === -1)
						if (bool) d.domains.push(domain)
					})
					//合并地址
					if (n.addresses && n.addresses.length) {
						if (d.addresses) {
							d.addresses.forEach(noe => {
								let i = n.addresses.findIndex(f => f.address === noe.address || noe.address.indexOf(f.address) > -1)
								if (i > -1) {
									//如果新数据有origin_id而原数据没有，用新数据
									if (!n.addresses[i].name && noe.name) n.addresses[i].name = noe.name
									if (!n.addresses[i].country && noe.country) n.addresses[i].country = noe.country
									if (!n.addresses[i].city && noe.city) n.addresses[i].city = noe.city
									if (!n.addresses[i].latitude && noe.latitude) n.addresses[i].latitude = noe.latitude
									if (!n.addresses[i].longitude && noe.longitude) n.addresses[i].longitude = noe.longitude
								} else {
									n.addresses.push(noe)
								}
							})
						}
					} else if (d.addresses) {
						n.addresses = d.addresses
					}
					n.stock = ff.util.arrayUnique([...d.stock, ...n.stock])
					n.imgs = ff.util.arrayUnique([...d.imgs,...n.imgs])

					if (!n.remark && d.remark) n.remark = d.remark
					return true
				}
			})

			if (!bool && d) {
				newDates.push(d)
			}
		}
		return newDates
	}

	pooledData<T extends keyof PooledDataParams>(datas: PooledDataParams[T], type: T, forcedMergers = false) {
		if (type === 'company') {
			return this.pooledCompanyData(<PooledDataParams['company']>datas, forcedMergers)
		} else {
			return this.pooledContactData(<PooledDataParams['contact']>datas)
		}
	}

	/**
	 * 判断一个字符串是否为职位名
	 * @param {string} str 需要判断的字符串
	 * @return {boolean} true:是职位名；false:不是职位名
	 * */
	isPosition(str: string): boolean {
		let p = [
			/Chairman/i,
			/President/i,
			/Founder/,
			/SVP/,
			/VP/,
			/Director/,
			/Supervisor/i,
			/Network Engineer/i,
			/猎头/,
			/Network Architect/i,
			/System Administrator/i,
			/Executive Assistant/i,
			/经理/,
			/运营/,
			/总监/,
			/董事长/,
			/执行董事/,
			/主席/,
			/首席/,
			/HR/,
			/高级产品/,
			/顾问/,
			/招聘/,
			/Maintenance/i,
			/recruitment officer/i,
			/ ?IT /,
			/CEO/i,
			/Chief Executive Officer/i,
			/CFO/i,
			/Chief Financial Officer/i,
			/COO/i,
			/Head of Operation/i,
			/Chief Operations Officer/i,
			/CIO/i,
			/Chief Information Officer/i,
			/CRO/i,
			/Chief Risk Officer/i,
			/CAO/i,
			/Chief Artistic Officer/i,
			/CBO/i,
			/Chief Brand Officer/i,
			/CCO/i,
			/Chief Cultural Officer/i,
			/CDO/i,
			/Chief Development Officer/i,
			/CFO/i,
			/Chief Finance Officer/i,
			/CHO/i,
			/Chief Human Resource Officer/i,
			/CKO/i,
			/Chief Knowledge Officer/i,
			/CMO/i,
			/Chief Marketing Officer/i,
			/CNO/i,
			/Chief Negotiation Officer/i,
			/CPO/i,
			/Chief Public Relation Officer/i,
			/CQO/i,
			/Chief Quality Officer/i,
			/CSO/i,
			/Chief Sales Officer/i,
			/CTO/i,
			/Chief Technology Officer/i,
			/CVO/i,
			/Chief Valuation Officer/i,
			/GM/i,
			/副总裁/i,
			/第一副总裁/i,
			/副总裁助理/i,
			/Human Resource Director/i,
			/OD/i,
			/MD/i,
			/Marketing Director/i,
			/OM/i,
			/PM/i,
			/PM/i,
			/PM/i,
			/BM/i,
			/DM/i,
			/RM/i,
			/AAD/i,
			/Associated Account Director/i,
			/AAD/i,
			/Associated Art Director/i,
			/副美术指导/i,
			/ACD/i,
			/Associated Creative Director/i,
			/AD/i,
			/Account Director/i,
			/业务指导/i,
			/AD/i,
			/Art Director/i,
			/美术指导/i,
			/AE/i,
			/Account Executive/i,
			/客户执行/,
			/客户服务/,
			/客户主任/,
			/AM/i,
			/AP/i,
			/Account Planner/i,
			/客户企划/i,
			/ASM/i,
			/Manager/i,
			/CD/i,
			/Creative Director/i,
			/Associate Project Manager/i,
			/创意指导/,
			/CGH/i,
			/Creative Group Head/i,
			/创意组长/i,
			/CW/i,
			/Copywriter/i,
			/撰稿人/i,
			/DCS/i,
			/Director Of Client Service/i,
			/客户主管/i,
			/ECD/i,
			/Executive Creative Director/i,
			/FA/i,
			/Finish Artist/i,
			/完稿、画师/i,
			/GAD/i,
			/Group Account Director/i,
			/GCD/i,
			/Group Creative Director/i,
			/GMD/i,
			/MD/i,
			/Media Director/i,
			/媒体指导/i,
			/SAD/i,
			/Senior Art Director/i,
			/高级美术指导/i,
			/SCW/i,
			/Senior Copywriter/i,
			/高级文案/i
		]

		return p.some(val => val.test(str))
	}

	/**
	 * 判断一个字符串是否为公司名
	 * @param {string} str 需要判断的字符串
	 * @return {boolean} true:是公司名；false:不是公司名
	 *
	 * */
	isCompany(str: string): boolean {
		let accounts = [/公司/, /company/, /[Cc]orporation/, /Co\., ?Ltd.?/, /[Ii]nternational/, /INC/, /[Ii]nstitute/]
		return accounts.some(val => val.test(str))
	}

	/**
	 * 拆分签名 分解开公司与职位
	 * @param {string} label 要分解的字符串
	 * @return {[string,string]} [{String}公司名,{String}职位名]
	 * */
	labelTreatment(label: string): [string, string] {
		//整理识别，公司和职位
		//@param {array} arr[string,string]
		//@return {array} [string(公司),string(职位)]
		let sorting = (arr: string[]) => {
			let p, a
			if (this.isPosition(arr[0])) {
				p = arr[0]
				a = arr[1]
			} else if (this.isCompany(arr[0])) {
				p = arr[1]
				a = arr[0]
			} else if (this.isPosition(arr[1])) {
				p = arr[1]
				a = arr[0]
			} else if (this.isCompany(arr[1])) {
				p = arr[0]
				a = arr[1]
			} else {
				a = arr[0]
				p = arr[1]
			}
			return [a, p]
		}

		let position = '',
				account = ''
		if (/ at /.test(label)) {
			let arr = label.split(' at ')
			position = arr[0]
			account = arr[1]
		} else if (/ ?[-—][-—]? ?/.test(label)) {
			let arr = label.split(/[-—][-—]?/)

			arr = arr.map(function (val) {
				return val ? val.replace(/^ | $/g, '') : ''
			})

			arr = sorting(arr)
			account = arr[0]
			position = arr[1]
		} else if (/，/.test(label)) {
			let arr = label.split(/[-—]/)

			arr = arr.map(function (val) {
				return val ? val.replace(/^ | $/g, '') : ''
			})

			if (arr[0].length === 1) {
			}
			arr = sorting(arr)
			account = arr[0]
			position = arr[1]
		} else {
			let isP = this.isPosition(label),
					isC = this.isCompany(label)

			if (isP && isC && label.split(/ +/).length === 2) {
				let arr = label.split(/ +/)
				arr = sorting(arr)
				account = arr[0]
				position = arr[1]
			} else if (this.isCompany(label)) {
				position = ''
				account = label
			} else if (this.isPosition(label)) {
				position = label
				account = ''
			} else {
				position = label
				account = ''
			}
		}
		account = account ? account : ''
		position = position ? position : ''

		account = ff.util.trim(account, ' ,的')
		position = ff.util.trim(position, ' ,的')

		if (account.length === 1) {
			account = ''
			position = label
		} else if (position.length === 1) {
			account = label
			position = ''
		}

		return [account, position]
	}
}

export default new CrawlerUtil()

export {CrawlerUtil}
