import * as config from './linkedin/lisn.config'
import Ajax from '../../common/ajax'

/**
 * @用途 通讯类，linkedin、facebook等通讯
 * @author 黄敏
 * @创建时间 2022-08-11 10:26
 **/

interface SendMessaging {
	code: number
	isFriend: boolean
	isPending?: boolean
}

interface CancelCb {
	(cancel: Axios.Canceler): void
}

export enum LinkedinAccount {
	nologin = 1, //未登录
	default, //普通账号
	sales //sales帐号
}

interface liErr {
	code: number
	msg: string
	time: number
	type: string
	conn_m: number
}

interface Stat {
	li_id: string | number
	li_uid?: string
	type: string
	code?: number
	msg?: string | object
}

interface Friend {
	id: number
	friend_status?: 0 | 1 | 2
}

export type GetLiStatData = {
	failure: {
		id: number
		type: string
		user_id: number
		li_id: string
		create_time: number
		code: number
		msg: string
		friend_status: number
		project_id: number
		li_uid: string
	}[]
} & {
	[key: string]: { [key: string]: number }
}

let ajax = new Ajax()

ajax.onCodeError((code: number, msg: string) => {
	if ([10, 11, 12, 14].includes(code)) {
		ff.gc.send('系统已退出登录', false)
	} else if ([40, 50].includes(code)) {
		ff.gc.send('系统已退出登录', true)
	}
})

let uidToUrnId: { [uid: string]: string } = {}

class Linkedin {
	readonly config = Object.freeze(config)
	readonly nologin = 1
	readonly default = 2
	readonly sales = 3

	login_status = this.nologin //登录状态，未登录，普通账号登录、lisn账号登录
	login_number_id = 0 //当前登录的linkedin数字ID


	//获取数据库的联系人
	async getDBContact(UID: string): Promise<Crawler.Contact | null> {
		//查详情本地数据库，是否有值
		let data = await ff.crawler.db.getContact(UID, 'linkedin-detail')
		if (data) return data

		//查基本本地数据库，是否有值
		data = await ff.crawler.db.getContact(UID, 'linkedin-base')
		return data
	}

	/**
	 * 获取linkedin的好友申请数量
	 * */
	async getInvitationCount() {
		let url = `https://www.linkedin.com/voyager/api/relationships/genericInvitationFacets?q=sent`
		await ff.util.asyncDelay(2000, 5000)

		let d: any = await ff.linkedin.ajax(url, {}, 'get')
		let p = d.elements.find((v: any) => v.invitationType === 'CONNECTION')
		return p ? p.count : 0
	}

	/**
	 * 获取linkedin的好友申请列表
	 * */
	async getInvitationList(page: number = 0, size = 10) {
		if (page < 0) return []
		let url = `https://www.linkedin.com/voyager/api/relationships/sentInvitationViewsV2?count=${size}&invitationType=CONNECTION&q=invitationType&start=${page}`
		let d: any = await ff.linkedin.ajax(url, {}, 'get')

		return d.elements
	}


	/*
	 * 获取联系人基本信息
	 * @param {String} linkedin ID
	 * @returns {Object} baseInfo|null
	 * */
	async getContactBase(UID: string): Promise<Crawler.Contact | null> {
		//查详情本地数据库，是否有值
		let data = await this.getDBContact(UID)
		if (data) return data

		await ff.util.asyncDelay(1500, 4000)

		//无值，向下执行
		//网络请求
		let url = `https://www.linkedin.com/voyager/api/identity/dash/profiles?q=memberIdentity&memberIdentity=${UID}&decorationId=com.linkedin.voyager.dash.deco.identity.profile.FullProfileWithEntities-91`
		let code = await ff.linkedin.ajax(url)

		if (code === null || !code.elements || !code.elements.length) return null
		let codeActive = code.elements[0]

		//联系人姓名
		let firstName = ff.util.trim(codeActive.firstName)
		let lastName = ff.util.trim(codeActive.lastName)
		let name = ff.util.getFullName(firstName, lastName)
		if (codeActive.maidenName) {
			name += ' | ' + codeActive.maidenName
		}

		let model = ff.crawler.util.contactModel({
			origin: 'linkedin',
			plugin_id: 'linkedin-' + codeActive.publicIdentifier,
			completed: false,
			update_time: Date.now()
		})

		model.contacts_names.push({first_name: firstName ? firstName : ' ', last_name: lastName ? lastName : ' '})

		// model.first_name = firstName
		// model.last_name = lastName

		let friendInfo = await ff.linkedin.getFriendInfo(UID)

		let OriginInfo: Crawler.ContactInfo = {
			type: 'linkedin',
			info: 'https://www.linkedin.com/in/' + codeActive.publicIdentifier,
			friend: friendInfo.isFriend ? 1 : 0,
			premium: codeActive.premium ? 1 : 0,
			unique_id: ff.linkedin.getLinkedinUniqueId(codeActive.objectUrn),
			origin_name: name
		}

		let works: Crawler.Work[] = [],
				edus: Crawler.Education[] = [],
				icon = ''

		//工作经历
		let companyView = codeActive.profilePositionGroups
		if (companyView && companyView.elements && companyView.elements.length) {
			companyView.elements.forEach((companyWorks: any, i: number) => {
				let workGroup = (companyWorks.profilePositionInPositionGroup && companyWorks.profilePositionInPositionGroup.elements) || []

				if (!workGroup.length) return
				let thisWorks: Crawler.Work[] = []

				workGroup.forEach((work: any) => {
					let start_date, end_date, origin_id

					//开始时间
					if (work.dateRange && work.dateRange.start) {
						start_date = {
							year: work.dateRange.start.year,
							month: work.dateRange.start.month
						}
					}

					//结束时间
					if (work.dateRange && work.dateRange.end) {
						end_date = {
							year: work.dateRange.end.year,
							month: work.dateRange.end.month
						}
					}

					if (work.companyUrn) {
						//urn:li:fsd_company:2606072
						let exec = /urn:li:fsd_company:(\d+)$/.exec(work.companyUrn)
						if (exec) {
							origin_id = ff.crawler.util.createUniqueId(exec[1], 'linkedin', 'company')
						}
					}

					let data: Crawler.Work = {
						location: work.geoLocationName || work.locationName,
						positions: work.title || '',
						company: work.companyName || '',
						start_date: start_date,
						end_date: end_date,
						descr: work.description || '',
						unique_id: origin_id,

						is_quit: false
					}

					//是否离职判断
					//有结束时间，为1已离职
					if (end_date) {
						data.is_quit = true
					} else if (start_date) {
						data.is_quit = false
					} else data.is_quit = i > 0

					if (data.company && data.company.length > 1) {
						thisWorks.push(data)
					}
				})

				works.push(...thisWorks)
			})
		}

		//教育经历
		let eduView = codeActive.profileEducations
		if (eduView && eduView.elements && eduView.elements.length) {
			eduView.elements.forEach((edu: any) => {
				let start_date, end_date

				//开始时间
				if (edu.dateRange && edu.dateRange.start) {
					start_date = {
						year: edu.dateRange.start.year,
						month: edu.dateRange.start.month || 0
					}
				}

				//结束时间
				if (edu.dateRange && edu.dateRange.end) {
					end_date = {
						year: edu.dateRange.end.year,
						month: edu.dateRange.end.month || 0
					}
				}

				let data = {
					college: edu.schoolName,
					degree_name: edu.degreeName,
					study: edu.fieldOfStudy,
					start_date: start_date,
					end_date: end_date
				}

				if (edu.schoolName && edu.schoolName.length > 1) {
					edus.push(data)
				}
			})
		}

		model.educations = edus

		if (codeActive.locationName) {
			model.geo_region = codeActive.locationName
		} else if (codeActive.geoLocation && codeActive.geoLocation.geo) {
			let geo = codeActive.geoLocation.geo

			if (geo.defaultLocalizedName) {
				model.geo_region = geo.defaultLocalizedName
			} else if (geo.country && geo.country.defaultLocalizedName) {
				model.geo_region = geo.country.defaultLocalizedName
			}
		}

		//国家
		let locationView = codeActive.primaryLocale
		let countryCode: any

		if (model.geo_region) {
			OriginInfo.origin_address = model.geo_region
		}

		if (locationView.location && locationView.location.countryCode) {
			countryCode = locationView.location.countryCode.toUpperCase()
		} else if (locationView && locationView.country) {
			countryCode = locationView.country.toUpperCase()
		}

		if (countryCode) {
			model.country = countryCode
		} else {
			model.country = model.geo_region
		}

		model.infos.push(OriginInfo)

		//获取头像
		let pictureView = codeActive.profilePicture
		if (pictureView && codeActive.profilePicture && codeActive.profilePicture.displayImageReference) {
			let d = codeActive.profilePicture.displayImageReference
			if (d.vectorImage) {
				icon = this.extractImageUrl(d.vectorImage)
			}
		}

		model.imgs = icon ? [icon] : []

		//签名
		let signature = codeActive.headline
		model.remark = codeActive.summary || ''

		//work字段可能为空或work字段都是离职状态
		//查找签名是否有公司名或职位
		if (!works.length || works.every(v => v.is_quit)) {
			let newWork = {
				positions: '',
				company: '',
				descr: '',
				is_quit: false
			}

			if (signature) {
				let label = ff.crawler.util.labelTreatment(signature)

				let company = label[0]
				let title = label[1]

				newWork.company = company
				newWork.positions = title
				if (newWork.company && newWork.company.length > 1) {
					works.unshift(newWork)
				}
			}
		}

		model.works = works

		if (codeActive.birthDateOn) {
			if (codeActive.birthDateOn.year) model.birth_year = codeActive.birthDateOn.year
			if (codeActive.birthDateOn.month) model.birth_month = codeActive.birthDateOn.month
			if (codeActive.birthDateOn.day) model.birth_day = codeActive.birthDateOn.day
		}

		ff.crawler.db.saveContact(UID, model, 'linkedin-base')
		return model
	}

	/**
	 * linkedin 好友邀请/发送消息上报
	 * */
	setStat(data: Stat | Friend) {
		if ('msg' in data && data.msg && typeof data.msg === 'object') {
			data.msg = JSON.stringify(data.msg)
		}
		return ajax.post('user/userStatisticsAdd', data)
	}

	/**
	 * linkedin 获取统计
	 * */
	getStat(li_id: string | number): Promise<GetLiStatData> {
		return ajax.post('user/userStatisticsList', {li_id: li_id})
	}

	/**
	 * @Description: 根据liID获取aaxa队列信息
	 * @author bear
	 * @date 2021/8/12
	 **/
	getDetail(li_uid: string) {
		return ajax.post('user/userStatisticsLog', {li_uid: li_uid})
	}

	get liErr() {
		return {
			get() {
				let liErr = localStorage.getItem('liErr') || '{"code":0,"msg":"","time":0,"type":""}'
				return JSON.parse(liErr)
			},
			set(liErr: liErr) {
				localStorage.setItem('liErr', JSON.stringify(liErr))
			},
			remove() {
				localStorage.removeItem('liErr')
			}
		}
	}

	/**
	 * 'https://www.linkedin.com/pub/vlad-cealicu/2/807/29a'
	 * linkedin 老格式转换，/pub/vlad-cealicu/2/807/29a
	 * */
	async pubToIn(url: string) {
		if (!/linkedin\.com\/pub\/.*/.test(url)) {
			return url
		}
		try {
			let e = await ff.axios(url)

			let uid = ff.contactInfo.getInfoId('linkedin', e.request.responseURL, false, true)
			if (uid) {
				return 'https://www.linkedin.com/in/' + uid
			} else {
				return ''
			}
		} catch (e) {
			ff.linkedin.ajaxErrorHandle(e)
			throw e
		}
	}


	/**
	 * 提取图片信息
	 * @param {{artifacts: any[]; rootUrl: string}} picture linkedin通用格式
	 * */
	extractImageUrl(picture: { artifacts: any[]; rootUrl: string }): string {
		let url = ''
		if (picture && picture.artifacts) {
			let imgs = picture.artifacts

			let maxWidth = 0
			let simg: any = null

			imgs.forEach(v => {
				if (v.width > maxWidth) {
					maxWidth = v.width
					simg = v
				}
			})

			let rootUrl = picture.rootUrl || ''
			let img = simg.fileIdentifyingUrlPathSegment || ''

			url = rootUrl + img
		}

		return url
	}

	/**
	 * 从linkedin的targetUrn中提取UniqueId
	 * targetUrn格式：urn:li:fs_miniProfile:ACoAAAqEjnEBNiMcjmpeCpYs09szr2LFNeFN8dk
	 * @param targetUrn
	 * @param type
	 * @return {String}
	 * */
	getLinkedinUniqueId(targetUrn: string, type: Crawler.DataType = 'contact'): string {
		let id = ff.linkedin.getEndId(targetUrn)
		if (id) {
			return ff.crawler.util.createUniqueId(id, 'linkedin', type)
		} else {
			return ''
		}
	}

	//获取linkedin登录凭证
	async getToken(): Promise<string> {
		let token = await ff.crawler.util.getCookie('JSESSIONID', 'linkedin.com')
		if (!token) {
			token = await ff.crawler.util.getCookie('JSESSIONID', 'www.linkedin.com')
		}
		if (!token) token = ''
		return ff.util.trim(token, '"')
	}

	/**
	 * 获取当前登录者 数字ID
	 * https://www.linkedin.com/psettings/policy/notices?types=COMMENTS_VISIBILITY
	 * */

	async getNumberId() {
		if (!this.login_number_id) {
			let url = 'https://www.linkedin.com/psettings/policy/notices?types=COMMENTS_VISIBILITY'
			let r = await this.ajax(url, {}, 'get')
			if (r && r.content) {
				let u = r.content.member,
						arr = u.split(':')
				this.login_number_id = Number(arr[arr.length - 1])
			}
		}

		return this.login_number_id
	}

	/**
	 * 获取linkedin的密文id
	 * @param {string} uid uid是连接中 https://www.linkedin.com/in/yourk-soboros-181b0a63 中的 yourk-soboros-181b0a63
	 * */
	async getUrnId(uid: string) {

		if (uidToUrnId[uid]) return uidToUrnId[uid]

		//获取用户联系方式
		let url = 'https://www.linkedin.com/voyager/api/identity/profiles/' + uid + '/profileContactInfo'
		//urn:li:fs_contactinfo:ACoAAA2DOHEBYUISXyhKvgYlzk0ko6Gsbj7PT7Y
		let d: { entityUrn: string } = await ff.linkedin.ajax(url);

		let urnid = d.entityUrn.replace('urn:li:fs_contactinfo:', '');

		uidToUrnId[uid] = urnid

		return urnid
	}

	ajaxErrorHandle(e: any) {
		if (e.response) {
			//没有登录
			if (e.response.status === 999 || e.response.status === 401) {
				console.warn('linkedin.ts')
				ff.gc.send('登录状态linkedin更新', 'nologin')
				throw ff.error('no_login')
			} else if (e.response.status === 403) {
				throw ff.error('no_access', 'no access: The linkedin may not exist, or the user may have restricted access')
			} else if (e.response.status === 404) {
				throw ff.error('not_found')
			} else if (e.response.status === 429) {
				let err_data = {code: e.response.status}
				if (e.response.data) err_data = {...err_data, ...e.response.data}
				throw ff.error('frequency_too_fast', undefined, err_data)
			} else if (e.response.status === 451) {
				throw ff.error('linkedin_vpn')
			}
		}
		if (e.message === 'Network Error') {
			//没有网络
			throw ff.error('network_error')
		}
	}

	//linkedin ajax请求
	//@param {String} url 请求地址
	//@param {Object} data 请求数据
	//@param {String} type 请求类型，get或post
	async ajax(url: string, data: { [key: string]: any } = {}, type: 'get' | 'post' = 'get', cancelCb?: CancelCb, headers?: { [key: string]: string | number }): Promise<any> {
		try {
			let token = await this.getToken()
			if (!token) throw ff.error('no_login')
			let fHead = headers ? headers : {}
			let option: Axios.RequestConfig = {
				timeout: 60000,
				method: type,
				url: url,
				withCredentials: true, //可跨域访问
				headers: {
					'X-Requested-With': 'XMLHttpRequest',
					'csrf-token': token,
					'x-restli-protocol-version': '2.0.0',
					...fHead
				},
				xsrfHeaderName: 'csrf-token', //  默认
				cancelToken: new ff.axios.CancelToken(function (cancel: Axios.Canceler) {
					if (cancelCb) {
						cancelCb(cancel)
					}
				})
			}
			if (type === 'post') {
				option.data = data
			} else {
				option.params = {...data, fms: '1'}
			}
			let res = await ff.axios(option)

			return res.data
		} catch (e: any) {
			this.ajaxErrorHandle(e)
			throw e
		}
	}

	/*
	 * 获取由冒号(:)分隔的字符串最后一段id
	 * 格式：urn:li:fs_miniProfile:ACoAAAqEjnEBNiMcjmpeCpYs09szr2LFNeFN8dk
	 * urn:li:fsd_company:10142069
	 * */
	getEndId(urn: string | undefined) {
		let id: string = ''
		if (typeof urn === 'string' && urn.indexOf(':') > 2) {
			let a = urn.split(':')
			id = a[a.length - 1]
		}
		return id
	}

	//检查是否是好友
	//@param uid string
	//@param noCache 不拿缓存数据
	//@return FriendInfo
	async getFriendInfo(uid: string, noCache?: boolean): Promise<Crawler.FriendInfo> {
		let uuid = await this.getNumberId()
		let friendInfo: Crawler.FriendInfo = {
			isFriend: false,
			isPending: false,
			encrypt_uid: '',
			uid: uuid
		}

		if (!noCache) {
			let d = await ff.crawler.db.getValidationFriend(uid, 'linkedin', uuid)
			if (d) return d
		}

		let url = `https://www.linkedin.com/voyager/api/identity/profiles/${uid}/profileActions?contextType=NON_SELF_PROFILE_VIEW&profileActionTypes=List(ACCEPT,BLOCK,BUILD_RESUME,CONNECT,DISCONNECT,FOLLOW,IGNORE,INVITATION_PENDING,PERSONALIZED_CONNECT,RECOMMEND,REPORT,REQUEST_RECOMMENDATION,SAVE_TO_PDF,SIGNUP,UNFOLLOW,VIEW_PROFILE_IN_RECRUITER,VIEW_PROFILE_IN_SALES_NAVIGATOR,APPRECIATE,COMPOSE_MESSAGE,ADD_TEAMMATE)`
		let data = await this.ajax(url)

		console.warn('验证好友接口', data)

		if (!data) {
			console.log('linkedin好友验证结果返回为空：可能验证出问题')
			return friendInfo
		}

		if (data.entityUrn) {
			friendInfo.encrypt_uid = this.getEndId(data.entityUrn)
		}

		let longKey = 'com.linkedin.voyager.identity.profile.actions.GenericProfileAction'
		let pending = 'com.linkedin.voyager.identity.profile.actions.InvitationPending'
		let connect = 'com.linkedin.voyager.identity.profile.actions.Connect'
		let disconnect = 'com.linkedin.voyager.identity.profile.actions.Disconnect'
		let pAction = data.primaryAction

		if (pAction && pAction.action) {
			if (pAction.action[longKey] && pAction.action[longKey].composeOption) {
				let compose = data.primaryAction.action[longKey].composeOption

				if (compose) {
					let type = compose.composeOptionType

					if (type === 'REPLY' || type === 'CONNECTION_MESSAGE') {
						friendInfo.isFriend = true
					}

					if (compose.composeNavigationContext && compose.composeNavigationContext.existingConversationUrn) {
						friendInfo.message_id = this.getEndId(compose.composeNavigationContext.existingConversationUrn)
					}
				}
			} else if (pAction.action[pending]) {
				friendInfo.isPending = true
			}
		}

		//overflowActions 用于显示操作按钮
		// com.linkedin.voyager.identity.profile.actions.InvitationPending 表示显示了pending按钮
		let oAction = data.overflowActions
		if (!friendInfo.isFriend && !friendInfo.isPending && oAction && oAction.length) {
			oAction.some((e: any) => {
				if (!e.action) return false
				if (e.action[pending]) {
					friendInfo.isPending = true
					friendInfo.isFriend = false
					return true
				} else if (e.action[connect]) {
					friendInfo.isPending = false
					friendInfo.isFriend = false
					return true
				} else if (e.action[disconnect]) {
					friendInfo.isPending = false
					friendInfo.isFriend = true
					return true
				}
			})
		}

		await ff.crawler.db.saveValidationFriend(uid, 'linkedin', friendInfo, uuid)
		return friendInfo
	}

	/**
	 * 撤回好友邀请
	 * https://www.linkedin.com/voyager/api/relationships/invitations?action=closeInvitations
	 * linkedin接口是单个的，这里传多个也能成功。
	 * */
	async cancelInviate(id: any, stat?: { msg: string | object; li_uid: string; type: string }[]) {
		let url = 'https://www.linkedin.com/voyager/api/relationships/invitations?action=closeInvitations'

		let ids = id.map((v: string) => {
			return {
				entityUrn: v, //"urn:li:fs_relInvitation:6774868122634739712",
				genericInvitation: false,
				genericInvitationType: 'CONNECTION'
			}
		})
		/*[{
						entityUrn: id,//"urn:li:fs_relInvitation:6774868122634739712",
						genericInvitation: false,
						genericInvitationType: "CONNECTION",
				}]*/
		let form = {
			inviteActionType: 'ACTOR_WITHDRAW',
			inviteActionData: ids
		}

		await this.ajax(url, form, 'post')
		if (stat?.length) {
			for (let n of stat) {
				await this.setLimit(n)
			}
		}
		return
	}

	/**
	 * 用手机发送好友邀请
	 * @li ID
	 * @aaxa 任务ID
	 */
	async sentInvite(uid: string, ext: { task_id: number; contact_id: number; info_id: number }) {
		let url = 'https://www.linkedin.com/mwlite/invite/'
		let form = {inviteeVanityName: uid, trackingId: 'bGZ5ncrPR5CPQlWktzebxw=='},
				head = {
					'User-Agent': 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Mobile Safari/537.36'
				}
		try {
			let r = await this.ajax(url, form, 'post', () => {
			}, {'f-head': JSON.stringify(head)})
			if (r?.inviteSuccess) {
				this.setLimit({li_uid: uid, type: 'connect_m', msg: JSON.stringify(ext)})
			} else {
				let err = {code: 449, msg: 'invite failed'}
				this.setLimit({li_uid: uid, type: 'connect_m', code: 449, msg: JSON.stringify({...ext, ...err})})
				this.liErr.remove()
				throw err
			}
		} catch (e: any) {
			let err = {
				code: e.data?.status || e.data?.code || e.response?.status || e.code,
				msg: e.message,
				data: e.data
			}
			if (err.code === 429) err.msg = 'You’ve reached the weekly invitation limit'
			if (err.code !== 449) {
				this.setLimit({
					li_uid: uid,
					type: 'connect_m',
					code: err.code,
					msg: JSON.stringify(err)
				})
			}
			this.liErr.remove()
			throw e
		}
	}

	async setLimit(data: { li_uid?: string; code?: number; type: 'connect' | 'connect_m' | string; msg: string | object }) {
		if ('msg' in data && data.msg && typeof data.msg === 'object') {
			data.msg = JSON.stringify(data.msg)
		}
		let liId = await this.getNumberId()
		let form = {li_id: liId, ...data}
		this.setStat(form)
	}

	async sendConnect(uid: string, profileId: string, content: string) {
		let url = 'https://www.linkedin.com/voyager/api/growth/normInvitations'

		let form = {
			emberEntityName: 'growth/invitation/norm-invitation',
			invitee: {'com.linkedin.voyager.growth.invitation.InviteeProfile': {profileId: profileId}},
			trackingId: '7QpjOBGYQhmKftte6WR2qQ==',
			message: content
		}
		await this.ajax(url, form, 'post')
		await this.setLimit({li_uid: uid, type: 'connect', msg: ''})

		await ff.util.asyncDelay(2000, 5000)

		let r = await this.getFriendInfo(uid, true)
		if (!r.isPending) {
			await ff.util.asyncDelay(3000, 5000)
			r = await this.getFriendInfo(uid, true)
		}
		return r
	}

	//没有messageid又是好友时，发送消息
	//@param encrypt_uid 加密的字符串id
	//@param centent 要发送的信息
	private async _noMidMessageing(encrypt_uid: string, centent: string) {
		console.log('没有id时发送')
		let url = 'https://www.linkedin.com/voyager/api/messaging/conversations?action=create'
		let form = {
			keyVersion: 'LEGACY_INBOX',
			conversationCreate: {
				eventCreate: {
					value: {
						'com.linkedin.voyager.messaging.create.MessageCreate': {
							attributedBody: {
								text: centent,
								attributes: []
							},
							attachments: []
						}
					}
				},
				recipients: [encrypt_uid],
				subtype: 'MEMBER_TO_MEMBER'
			}
		}

		await this.ajax(url, form, 'post')
	}

	/*
	 * 发送消息给指定linkedin联系人
	 * @param {String} uid
	 * @param {String} 内容
	 * @returns {Object} {code:0,isFriend:'是否是好友，是true,不是false'}
	 * */
	async sendMessaging(uid: string, centent: string, contactOrigins?: Crawler.FriendInfo | null): Promise<SendMessaging> {
		//throw ff.error({code: 6000, msg: 'Failed to send'});
		if (!contactOrigins) contactOrigins = await this.getFriendInfo(uid)
		if (contactOrigins.isPending) {
			return {code: 0, isFriend: false, isPending: true}
		}
		let uo = {uid: uid}
		if (!contactOrigins.isFriend) {
			try {
				let r = await this.sendConnect(uid, contactOrigins.encrypt_uid, centent)
				if (!r || !r.isPending) {
					throw ff.error('common_error', 'Failed to send, Please try to access linkedin url is normal', r)
				} else {
					return {code: 0, isFriend: false, isPending: r.isPending}
				}
			} catch (e: any) {
				let code = e.data?.status || e.response?.status || e.code
				if (code === 429) {
					e.message = 'You’ve reached the weekly invitation limit'
				}
				let msg = {code: e.code, msg: e.message, ...uo, ...(e.data ? e.data : {})}
				this.setLimit({li_uid: uid, type: 'connect', code: code, msg: msg})
				throw e
			}
		}

		try {
			let url: string = '',
					form: any
			if (contactOrigins.message_id) {
				url = `https://www.linkedin.com/voyager/api/messaging/conversations/${contactOrigins.message_id}/events?action=create`
				form = {
					eventCreate: {
						value: {
							'com.linkedin.voyager.messaging.create.MessageCreate': {
								attributedBody: {
									text: centent,
									attributes: []
								},
								attachments: []
							}
						}
					},
					dedupeByClientGeneratedToken: false
				}
				await this.ajax(url, form, 'post')
			} else {
				await this._noMidMessageing(contactOrigins.encrypt_uid, centent)
				await ff.crawler.db.delValidationFriend(uid, 'linkedin')
			}
			return {code: 0, isFriend: true}
		} catch (e: any) {
			if (e.response && e.response.status === 500) {
				await this._noMidMessageing(contactOrigins.encrypt_uid, centent)
				await ff.crawler.db.delValidationFriend(uid, 'linkedin')
				return {code: 0, isFriend: true}
			} else {
				throw e
			}
		}
	}

	/*
	* 获取connections的列表
	* 默认为第一页
	* */
	async getConnections(url?: string): Promise<Crawler.Result> {
		let result: Crawler.Result = {
			contacts: [],
			companys: []
		}

		if (!url)
			url =
					'https://www.linkedin.com/voyager/api/relationships/dash/connections?count=40&decorationId=com.linkedin.voyager.dash.deco.web.mynetwork.ConnectionListWithProfile-5&q=search&sortType=RECENTLY_ADDED'

		let api = url.replace(/&start=\d+/, '')

		let data = await ff.linkedin.ajax(url)

		if (!data || !data.elements || !data.elements.length) return result

		let total = data.paging.total || data.paging.count

		let batch: Crawler.Paging = {
			title: 'linkedin connections',
			api: api,
			uid: api,
			method: 'ff.linkedin.getConnections',
			param_start: 0,
			total: total,
			param_step: data.paging.count,
			param_name: 'start',
			page_size: data.paging.count
		}

		result.paging = batch

		data.elements.forEach((c: any) => {
			let profile = c.connectedMemberResolutionResult
			if (!profile) return
			let firstName = ff.util.trim(profile.firstName)
			let lastName = ff.util.trim(profile.lastName)
			let name = ff.util.getFullName(firstName, lastName)

			let contact = ff.crawler.util.contactModel({
				origin: 'linkedin',
				update_time: Date.now(),
				// name: name,
				completed: false,
				plugin_id: 'linkedin-' + profile.publicIdentifier
			})
			contact.contacts_names.push({first_name: firstName, last_name: lastName})
			contact.infos.push({
				type: 'linkedin',
				info: `https://www.linkedin.com/in/${profile.publicIdentifier}`,
				unique_id: this.getLinkedinUniqueId(profile.entityUrn),
				friend: 1
			})

			//工作经历
			if (profile.headline) {
				let newWork = {
					positions: '',
					company: '',
					descr: '',
					is_quit: false
				}

				let label = ff.crawler.util.labelTreatment(profile.headline)
				let company = label[0]
				let title = label[1]
				newWork.company = company || name
				newWork.positions = title

				if (newWork.company.length > 1) {
					contact.works.push(newWork)
				}
			}

			if (profile?.profilePicture?.displayImageReference?.vectorImage) {
				let url = this.extractImageUrl(profile.profilePicture.displayImageReference.vectorImage)
				if (url) contact.imgs.push(url)
			}

			result.contacts?.push(contact)
		})

		return result
	}

}


export default  Linkedin
